// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:checked_yaml/checked_yaml.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/config/ios_config.dart';
import 'package:yaml/yaml.dart';

@usecase
class GetIosConfigUsecase extends AsyncUseCase<String, IosConfig> {
  const GetIosConfigUsecase() : super();

  @override
  FutureOr<Res<IosConfig>> execute(String params) => unsafe(() {
        final file = File(params);
        final content = file.readAsStringSync();

        return checkedYamlDecode(
          content,
          (json) => IosConfig.fromJson(json!['ios'] as YamlMap),
        );
      });
}
