// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/core/enums/operation_type.dart';

/// Defines every write operation in CRUD.
mixin CreateOperation<In, Out> on AsyncUseCase<In, Out> {
  /// {@macro create}
  OperationType get operationType => OperationType.create;
}

/// Defines every read operation in CRUD.
mixin ReadOperation<In, Out> on AsyncUseCase<In, Out> {
  /// {@macro read}
  OperationType get operationType => OperationType.read;
}

/// Defines every update operation in CRUD.
mixin UpdateOperation<In> on AsyncUseCase<In, void> {
  /// {@macro update}
  OperationType get operationType => OperationType.update;
}

/// Defines every delete operation in CRUD.
mixin DeleteOperation<In> on AsyncUseCase<In, void> {
  /// {@macro delete}
  OperationType get operationType => OperationType.delete;
}
