// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:go_router/go_router.dart';
import 'package:wyatt_go_router/src/core/enums/page_protection.dart';

/// Defines if a GoRoute is public or not.
///
/// By default, all routes are in the [PageProtection.none] state.
extension GoRouteExtension on GoRoute {
  static final _protection = Expando<PageProtection>();

  /// Returns `true` if the route is public.
  bool get isPublic => _protection[this] == PageProtection.public;

  /// Returns `true` if the route is protected.
  bool get isProtected => _protection[this] == PageProtection.protected;

  /// Returns `true` if the route is neither public nor protected.
  /// This is the default state.
  bool get isNone => _protection[this] == PageProtection.none;

  /// Sets the route to be public.
  /// This is useful for routes that should be accessible
  /// without authentication.
  /// ```dart
  /// GoRoute(
  ///  path: '/sign_in',
  ///  ...
  /// )..setPublic(),
  /// ```
  void setPublic() => _protection[this] = PageProtection.public;

  /// Sets the route to be protected.
  /// This is useful for routes that should only be accessible
  /// with authentication.
  void setProtected() => _protection[this] = PageProtection.protected;

  PageProtection get protection => _protection[this] ?? PageProtection.none;
}
